from typing import Dict, List, Optional

from llm_guard.transformers_helpers import pipeline_zero_shot_classification
from llm_guard.util import logger

from .base import Scanner

MODEL_BASE = {
    "path": "MoritzLaurer/deberta-v3-base-zeroshot-v1",
    "onnx_path": "laiyer/deberta-v3-base-zeroshot-v1-onnx",
    "max_length": 512,
}
MODEL_LARGE = {
    "path": "MoritzLaurer/deberta-v3-large-zeroshot-v1",
    "onnx_path": "laiyer/deberta-v3-large-zeroshot-v1-onnx",
    "max_length": 512,
}
ALL_MODELS = [MODEL_BASE, MODEL_LARGE]


class BanTopics(Scanner):
    """
    BanTopics class is used to ban certain topics from the prompt.

    It uses a HuggingFace model to perform zero-shot classification.
    """

    def __init__(
        self,
        topics=List[str],
        threshold: float = 0.6,
        model: Optional[Dict] = None,
        use_onnx: bool = False,
    ):
        """
        Initialize BanTopics object.

        Args:
            topics (List[str]): List of topics to ban.
            threshold (float, optional): Threshold to determine if a topic is present in the prompt. Default is 0.75.
            model (Dict, optional): Model to use for zero-shot classification. Default is deberta-v3-base-zeroshot-v1.
            use_onnx (bool, optional): Whether to use ONNX for inference. Default is False.

        Raises:
            ValueError: If no topics are provided.
        """
        if len(topics) == 0:
            raise ValueError("No topics provided")

        if model is None:
            model = MODEL_BASE

        if model not in ALL_MODELS:
            raise ValueError(f"Model must be in the list of allowed: {ALL_MODELS}")

        self._topics = topics
        self._threshold = threshold

        self._classifier = pipeline_zero_shot_classification(
            model=model["path"],
            onnx_model=model["onnx_path"],
            use_onnx=use_onnx,
            max_length=model["max_length"],
            truncation=True,
        )

    def scan(self, prompt: str) -> (str, bool, float):
        if prompt.strip() == "":
            return prompt, True, 0.0

        output_model = self._classifier(prompt, self._topics, multi_label=False)

        max_score = round(max(output_model["scores"]) if output_model["scores"] else 0, 2)
        if max_score > self._threshold:
            logger.warning(
                f"Topics detected for the prompt {output_model['labels']} with scores: {output_model['scores']}"
            )

            return prompt, False, max_score

        logger.debug(
            f"No banned topics detected ({output_model['labels']}, scores: {output_model['scores']})"
        )

        return prompt, True, 0.0



import re
from typing import List, Optional

from llm_guard.transformers_helpers import pipeline_text_classification
from llm_guard.util import logger

from .base import Scanner

_model_path = (
    "huggingface/CodeBERTa-language-id",
    "laiyer/CodeBERTa-language-id-onnx",  # ONNX model
)

SUPPORTED_LANGUAGES = ["go", "java", "javascript", "php", "python", "ruby"]


class Code(Scanner):
    """
    A class for scanning if the prompt includes code in specific programming languages.

    This class uses the transformers library to detect code snippets in the output of the language model.
    It can be configured to allow or deny specific programming languages.
    """

    def __init__(
        self,
        allowed: Optional[List[str]] = None,
        denied: Optional[List[str]] = None,
        threshold: float = 0.5,
        use_onnx: bool = False,
    ):
        """
        Initializes Code with the allowed and denied languages.

        Parameters:
            allowed (Optional[List[str]]): A list of allowed languages. Default is an empty list.
            denied (Optional[List[str]]): A list of denied languages. Default is an empty list.
            threshold (float): The threshold for the risk score. Default is 0.5.
            use_onnx (bool): Whether to use ONNX for inference. Default is False.

        Raises:
            ValueError: If both 'allowed' and 'denied' lists are provided or if both are empty.
        """
        if not allowed:
            allowed = []

        if not denied:
            denied = []

        if len(allowed) > 0 and len(denied) > 0:
            raise ValueError("Provide either allowed or denied programming languages")

        if len(allowed) == 0 and len(denied) == 0:
            raise ValueError("No allowed or denied programming languages provided")

        if len(allowed) > 0 and not set(allowed).issubset(set(SUPPORTED_LANGUAGES)):
            raise ValueError(f"Allowed languages must be a subset of {SUPPORTED_LANGUAGES}")

        if len(denied) > 0 and not set(denied).issubset(set(SUPPORTED_LANGUAGES)):
            raise ValueError(f"Denied languages must be a subset of {SUPPORTED_LANGUAGES}")

        self._allowed = allowed
        self._denied = denied
        self._threshold = threshold

        self._pipeline = pipeline_text_classification(
            model=_model_path[0], onnx_model=_model_path[1], use_onnx=use_onnx, truncation=True
        )

        self._fenced_code_regex = re.compile(r"```(?:[a-zA-Z0-9]*\n)?(.*?)```", re.DOTALL)
        self._inline_code_regex = re.compile(r"`(.*?)`")

    def _extract_code_blocks(self, markdown: str) -> List[str]:
        # Extract fenced code blocks (between triple backticks)
        fenced_code_blocks = [
            block.strip() for block in self._fenced_code_regex.findall(markdown) if block.strip()
        ]

        # Extract inline code (between single backticks)
        inline_code = [
            code.strip()
            for code in self._inline_code_regex.findall(markdown)
            if code.strip() and any(char in code for char in "{}[]()=+-*/<>!")
        ]

        return fenced_code_blocks + inline_code

    def scan(self, prompt: str) -> (str, bool, float):
        if prompt.strip() == "":
            return prompt, True, 0.0

        # Try to extract code snippets from Markdown
        code_blocks = self._extract_code_blocks(prompt)
        if len(code_blocks) == 0:
            logger.debug("No Markdown code blocks found in the output")
            return prompt, True, 0.0

        logger.debug(f"Code blocks found in the output: {code_blocks}")

        # Only check when the code is detected
        for code_block in code_blocks:
            languages = self._pipeline(code_block)
            logger.debug(f"Detected languages {languages} in the block {code_block}")

            for language in languages:
                language_name = language["label"]
                score = round(language["score"], 2)

                if score < self._threshold:
                    continue

                if len(self._allowed) > 0 and language_name in self._allowed:
                    logger.debug(
                        f"Language {language_name} found in the allowed list with score {score}"
                    )
                    return prompt, True, 0.0

                if len(self._denied) > 0 and language_name in self._denied:
                    logger.warning(f"Language {language_name} is not allowed (score {score})")
                    return prompt, False, score

        if len(self._allowed) > 0:
            logger.warning(f"No allowed languages detected")
            return prompt, False, 1.0

        logger.debug(f"No denied languages detected")

        return prompt, True, 0.0



import importlib
from functools import lru_cache
from typing import Optional

from .util import device, lazy_load_dep, logger


@lru_cache(maxsize=None)  # Set maxsize=None for an unbounded cache
def get_tokenizer(model_identifier: str):
    """
    This function loads a tokenizer given a model identifier and caches it.
    Subsequent calls with the same model_identifier will return the cached tokenizer.

    Args:
        model_identifier (str): The model identifier to load the tokenizer for.
    """
    transformers = lazy_load_dep("transformers")
    tokenizer = transformers.AutoTokenizer.from_pretrained(model_identifier)
    return tokenizer


@lru_cache(maxsize=None)  # Unbounded cache
def is_onnx_supported() -> bool:
    is_supported = (
        str(device()) == "cpu" and importlib.util.find_spec("optimum.onnxruntime") is not None
    )
    if not is_supported:
        logger.warning(
            "Device is not CPU or ONNX Runtime is not available. "
            "Please install optimum: `pip install onnx onnxruntime optimum[onnx-runtime]` to enable ONNX Runtime optimizations."
        )

    return is_supported


def get_tokenizer_and_model_for_classification(
    model: str, onnx_model: Optional[str] = None, use_onnx: bool = False, **kwargs
):
    """
    This function loads a tokenizer and model given a model identifier and caches them.
    Subsequent calls with the same model_identifier will return the cached tokenizer.

    Args:
        model (str): The model identifier to load the tokenizer and model for.
        onnx_model (Optional[str]): The model identifier to load the ONNX model for. Defaults to None.
        use_onnx (bool): Whether to use the ONNX version of the model. Defaults to False.
        **kwargs: Keyword arguments to pass to the tokenizer and model.
    """
    tf_tokenizer = get_tokenizer(model)
    transformers = lazy_load_dep("transformers")

    if use_onnx and is_onnx_supported() is False:
        logger.warning("ONNX is not supported on this machine. Using PyTorch instead of ONNX.")
        use_onnx = False

    if use_onnx:
        if onnx_model is not None:
            model = onnx_model

        # Hack for some models
        tf_tokenizer.model_input_names = ["input_ids", "attention_mask"]

        optimum_onnxruntime = lazy_load_dep("optimum.onnxruntime", "optimum[onnxruntime]")
        tf_model = optimum_onnxruntime.ORTModelForSequenceClassification.from_pretrained(
            model, export=onnx_model is None
        )
        logger.debug(f"Initialized classification ONNX model {model} on device {device()}")
    else:
        tf_model = transformers.AutoModelForSequenceClassification.from_pretrained(model)
        logger.debug(f"Initialized classification model {model} on device {device()}")

    if kwargs.get("max_length", None) is None:
        kwargs["max_length"] = tf_tokenizer.model_max_length

    return tf_tokenizer, tf_model


def pipeline_text_classification(
    model: str, onnx_model: Optional[str] = None, use_onnx: bool = False, **kwargs
):
    transformers = lazy_load_dep("transformers")
    tf_tokenizer, tf_model = get_tokenizer_and_model_for_classification(
        model, onnx_model, use_onnx, **kwargs
    )

    return transformers.pipeline(
        "text-classification",
        model=tf_model,
        tokenizer=tf_tokenizer,
        device=device(),
        batch_size=1,
        **kwargs,
    )


def pipeline_zero_shot_classification(
    model: str, onnx_model: Optional[str] = None, use_onnx: bool = False, **kwargs
):
    transformers = lazy_load_dep("transformers")
    tf_tokenizer, tf_model = get_tokenizer_and_model_for_classification(
        model, onnx_model, use_onnx, **kwargs
    )

    return transformers.pipeline(
        "zero-shot-classification",
        model=tf_model,
        tokenizer=tf_tokenizer,
        device=device(),
        batch_size=1,
        **kwargs,
    )


def pipeline_ner(model: str, onnx_model: Optional[str] = None, use_onnx: bool = False, **kwargs):
    transformers = lazy_load_dep("transformers")
    tf_tokenizer = get_tokenizer(model)

    if use_onnx and is_onnx_supported() is False:
        logger.warning("ONNX is not supported on this machine. Using PyTorch instead of ONNX.")
        use_onnx = False

    if use_onnx:
        if onnx_model is not None:
            model = onnx_model

        optimum_onnxruntime = lazy_load_dep("optimum.onnxruntime", "optimum[onnxruntime]")
        tf_model = optimum_onnxruntime.ORTModelForTokenClassification.from_pretrained(
            model, export=onnx_model is None
        )
        logger.debug(f"Initialized ner pipeline for ONNX model {model} on device {device()}")
    else:
        tf_model = transformers.AutoModelForTokenClassification.from_pretrained(model)
        logger.debug(f"Initialized ner pipeline for model {model} on device {device()}")

    return transformers.pipeline(
        "ner",
        model=tf_model,
        tokenizer=tf_tokenizer,
        device=device(),
        batch_size=1,
        **kwargs,
    )



from typing import List, Optional

from llm_guard.input_scanners.regex import Regex as InputRegex

from .base import Scanner


class Regex(Scanner):
    """
    A class used to detect patterns in the output of a language model using regular expressions.

    This class uses two lists of regular expressions: good_patterns and bad_patterns. If good_patterns are
    provided, the output is considered valid if any good_pattern matches the output. Conversely, if
    bad_patterns are provided, the output is considered invalid if any bad_pattern matches the output.
    """

    def __init__(
        self,
        good_patterns: Optional[List[str]] = None,
        bad_patterns: Optional[List[str]] = None,
        redact=True,
    ):
        """
        Initializes an instance of the Regex class.

        Parameters:
            good_patterns (Optional[List[str]]): List of regular expression patterns that the output should match.
            bad_patterns (Optional[List[str]]): List of regular expression patterns that the output should not match.
            redact (bool): Whether to redact the output or not.

        Raises:
            ValueError: If no patterns provided or both good and bad patterns provided.
        """

        self._scanner = InputRegex(good_patterns, bad_patterns, redact)

    def scan(self, prompt: str, output: str) -> (str, bool, float):
        return self._scanner.scan(output)



from llm_guard.transformers_helpers import pipeline_text_classification
from llm_guard.util import calculate_risk_score, logger

from .base import Scanner

_model_path = (
    "unitary/unbiased-toxic-roberta",
    "laiyer/unbiased-toxic-roberta-onnx",  # ONNX model
)
_toxic_labels = [
    "toxicity",
    "severe_toxicity",
    "obscene",
    "threat",
    "insult",
    "identity_attack",
]


class Toxicity(Scanner):
    """
    A toxicity scanner that uses a pretrained Hugging Face model to assess the toxicity of a given text.

    If the toxicity score is less than a predefined threshold, the text is considered non-toxic. Otherwise, it is
    considered toxic.
    """

    def __init__(self, threshold: float = 0.7, use_onnx: bool = False):
        """
        Initializes Toxicity with a threshold for toxicity.

        Parameters:
           threshold (float): Threshold for toxicity. Default is 0.7.
           use_onnx (bool): Whether to use ONNX for inference. Default is False.

        Raises:
           None.
        """

        self._threshold = threshold
        self._pipeline = pipeline_text_classification(
            model=_model_path[0],
            onnx_model=_model_path[1],
            top_k=None,
            use_onnx=use_onnx,
            truncation=True,
        )

    def scan(self, prompt: str) -> (str, bool, float):
        if prompt.strip() == "":
            return prompt, True, 0.0

        results = self._pipeline(prompt)

        highest_toxicity_score = 0.0
        toxicity_above_threshold = []
        for result in results[0]:
            if result["label"] not in _toxic_labels:
                continue

            if result["score"] > self._threshold:
                toxicity_above_threshold.append(result)
            if result["score"] > highest_toxicity_score:
                highest_toxicity_score = result["score"]

        if len(toxicity_above_threshold) > 0:
            logger.warning(f"Detected toxicity in the text: {toxicity_above_threshold}")

            return prompt, False, calculate_risk_score(highest_toxicity_score, self._threshold)

        logger.debug(f"Not toxicity found in the text. Results: {results}")

        return prompt, True, 0.0



from typing import Dict, List, Optional

from llm_guard.transformers_helpers import pipeline_text_classification
from llm_guard.util import calculate_risk_score, logger

from .base import Scanner

# This model has been known to yield high false positive rates and might not be suited for production use.
MODEL_DEEPSET = {
    "path": "deepset/deberta-v3-base-injection",
    "onnx_path": "laiyer/deberta-v3-base-injection-onnx",
    "label": "INJECTION",
    "max_length": 512,
}

# This dataset is more up-to-date. However, it performs slower because based on RoBERTa-large model.
MODEL_GPTFUZZ = {
    "path": "hubert233/GPTFuzz",
    "onnx_path": "laiyer/GPTFuzz-onnx",
    "label": 1,
    "max_length": 512,
}

ALL_MODELS = [
    MODEL_DEEPSET,
    MODEL_GPTFUZZ,
]


class PromptInjection(Scanner):
    """
    A prompt injection scanner based on HuggingFace model. It is used to
    detect if a prompt is attempting to perform an injection attack.
    """

    def __init__(
        self, models: Optional[List[Dict]] = None, threshold: float = 0.9, use_onnx: bool = False
    ):
        """
        Initializes PromptInjection with a threshold.

        Parameters:
            models (List[Dict], optional): Chosen models to classify prompt. Default is JasperLS.
            threshold (float): Threshold for the injection score. Default is 0.75.
            use_onnx (bool): Whether to use ONNX for inference. Defaults to False.

        Raises:
            ValueError: If non-existent models were provided.
        """

        self._threshold = threshold

        if models is None:
            models = [MODEL_DEEPSET]

        for model in models:
            if model not in ALL_MODELS:
                raise ValueError(f"Models must be in the list of allowed: {ALL_MODELS}")

        logger.debug(f"Prompt injection models: {models}")

        pipelines = {}
        for model in models:
            try:
                pipelines[model["path"]] = pipeline_text_classification(
                    model=model["path"],
                    use_onnx=use_onnx,
                    onnx_model=model["onnx_path"],
                    truncation=True,
                    max_length=model["max_length"],
                )
            except Exception as err:
                logger.error(f"Failed to load model: {err}")

        self._text_classification_pipelines = pipelines

    def scan(self, prompt: str) -> (str, bool, float):
        if prompt.strip() == "":
            return prompt, True, 0.0

        for model_path in self._text_classification_pipelines:
            try:
                result = self._text_classification_pipelines[model_path](prompt)
            except Exception as err:
                logger.error(f"Failed to get output of the model: {err}")
                return prompt, True, 0.0

            model_label = ALL_MODELS[[m["path"] for m in ALL_MODELS].index(model_path)]["label"]

            injection_score = round(
                result[0]["score"] if result[0]["label"] == model_label else 1 - result[0]["score"],
                2,
            )

            if injection_score > self._threshold:
                logger.warning(
                    f"Detected prompt injection using {model_path} with score: {injection_score}"
                )

                return prompt, False, calculate_risk_score(injection_score, self._threshold)

            logger.debug(
                f"No prompt injection detected using {model_path}, score: {injection_score}"
            )

        return prompt, True, 0.0



from llm_guard.input_scanners.toxicity import Toxicity as InputToxicity

from .base import Scanner


class Toxicity(Scanner):
    """
    A class used to detect toxicity in the output of a language model.

    This class uses a pre-trained toxicity model from HuggingFace to calculate a toxicity score for the output.
    """

    def __init__(self, threshold: float = 0.7, use_onnx: bool = False):
        """
        Initializes an instance of the Toxicity class.

        Parameters:
            threshold (float): The threshold used to determine toxicity. Defaults to 0.7.
            use_onnx (bool): Whether to use ONNX for inference. Defaults to False.
        """

        self._scanner = InputToxicity(threshold, use_onnx)

    def scan(self, prompt: str, output: str) -> (str, bool, float):
        return self._scanner.scan(output)



from typing import Dict, List

import spacy
from presidio_analyzer import (
    AnalyzerEngine,
    EntityRecognizer,
    Pattern,
    PatternRecognizer,
    RecognizerRegistry,
)
from presidio_analyzer.nlp_engine import NlpEngine, NlpEngineProvider

from .ner_mapping import ALL_RECOGNIZER_CONF
from .transformers_recognizer import TransformersRecognizer


def _add_recognizers(
    registry: RecognizerRegistry, regex_groups, custom_names
) -> RecognizerRegistry:
    """
    Create a RecognizerRegistry and populate it with regex patterns and custom names.

    Args:
        regex_groups: List of regex patterns.
        custom_names: List of custom names to recognize.

    Returns:
        RecognizerRegistry: A RecognizerRegistry object loaded with regex and custom name recognizers.
    """

    if len(custom_names) > 0:
        registry.add_recognizer(
            PatternRecognizer(supported_entity="CUSTOM", deny_list=custom_names)
        )

    for pattern_data in regex_groups:
        label = pattern_data["name"]
        compiled_patterns = pattern_data["expressions"]
        patterns = []
        for pattern in compiled_patterns:
            patterns.append(Pattern(name=label, regex=pattern, score=pattern_data["score"]))
        registry.add_recognizer(
            PatternRecognizer(
                supported_entity=label,
                patterns=patterns,
                context=pattern_data["context"],
            )
        )

    return registry


def _get_nlp_engine() -> NlpEngine:
    # Use small spacy model, for faster inference.
    if not spacy.util.is_package("en_core_web_sm"):
        spacy.cli.download("en_core_web_sm")

    configuration = {
        "nlp_engine_name": "spacy",
        "models": [{"lang_code": "en", "model_name": "en_core_web_sm"}],
    }

    return NlpEngineProvider(nlp_configuration=configuration).create_engine()


def get_transformers_recognizer(recognizer_conf: Dict, use_onnx: bool = False) -> EntityRecognizer:
    if recognizer_conf not in ALL_RECOGNIZER_CONF:
        raise ValueError("Recognizer configuration is not found")

    model_path = recognizer_conf.get("DEFAULT_MODEL_PATH")
    supported_entities = recognizer_conf.get("PRESIDIO_SUPPORTED_ENTITIES")
    transformers_recognizer = TransformersRecognizer(
        model_path=model_path,
        supported_entities=supported_entities,
    )
    transformers_recognizer.load_transformer(use_onnx=use_onnx, **recognizer_conf)
    return transformers_recognizer


def get_analyzer(
    recognizer: EntityRecognizer, regex_groups, custom_names: List[str]
) -> AnalyzerEngine:
    nlp_engine = _get_nlp_engine()

    registry = RecognizerRegistry()
    registry.load_predefined_recognizers(nlp_engine=nlp_engine)
    registry = _add_recognizers(registry, regex_groups, custom_names)
    registry.add_recognizer(recognizer)
    registry.remove_recognizer("SpacyRecognizer")

    return AnalyzerEngine(nlp_engine=nlp_engine, registry=registry, supported_languages=["en"])



from llm_guard.transformers_helpers import pipeline_text_classification
from llm_guard.util import logger

from .base import Scanner

_model_path = (
    "valurank/distilroberta-bias",
    "laiyer/distilroberta-bias-onnx",  # ONNX model
)


class Bias(Scanner):
    """
    This class is designed to detect and evaluate potential biases in text using a pretrained model from HuggingFace.
    """

    def __init__(self, threshold: float = 0.75, use_onnx: bool = False):
        """
        Initializes the Bias scanner with a probability threshold for bias detection.

        Parameters:
           threshold (float): The threshold above which a text is considered biased.
                              Default is 0.75.
           use_onnx (bool): Whether to use ONNX instead of PyTorch for inference.
        """
        self._threshold = threshold

        self._classifier = pipeline_text_classification(
            model=_model_path[0],
            onnx_model=_model_path[1],
            truncation=True,
            use_onnx=use_onnx,
        )

    def scan(self, prompt: str, output: str) -> (str, bool, float):
        if output.strip() == "":
            return output, True, 0.0

        classifier_output = self._classifier(output)
        score = round(
            classifier_output[0]["score"]
            if classifier_output[0]["label"] == "BIASED"
            else 1 - classifier_output[0]["score"],
            2,
        )
        if score > self._threshold:
            logger.warning(
                f"Detected biased text with score: {score}, threshold: {self._threshold}"
            )

            return output, False, score

        logger.debug(f"Not biased result. Max score: {score}, threshold: {self._threshold}")

        return output, True, 0.0



from typing import List, Optional

from llm_guard.input_scanners.code import Code as InputCode

from .base import Scanner


class Code(Scanner):
    """
    A class for scanning if the model output includes code in specific programming languages.

    This class uses the transformers library to detect code snippets in the output of the language model.
    It can be configured to allow or deny specific programming languages.
    """

    def __init__(
        self,
        allowed: Optional[List[str]] = None,
        denied: Optional[List[str]] = None,
        threshold: float = 0.5,
        use_onnx: bool = False,
    ):
        """
        Initializes an instance of the Code class.

        Parameters:
            allowed (Optional[List[str]]): A list of programming languages that are allowed in the model output.
            denied (Optional[List[str]]): A list of programming languages that are not allowed in the model output.
            threshold (float): The threshold for the model output to be considered valid. Default is 0.5.
            use_onnx (bool): Whether to use ONNX for inference. Default is False.

        Raises:
            ValueError: If both 'allowed' and 'denied' lists are provided or if both are empty.
        """

        self._scanner = InputCode(allowed, denied, threshold, use_onnx)

    def scan(self, prompt: str, output: str) -> (str, bool, float):
        return self._scanner.scan(output)



import re
from typing import List

from llm_guard.transformers_helpers import pipeline_text_classification
from llm_guard.util import calculate_risk_score, logger

from .base import Scanner

_model_path = (
    "DunnBC22/codebert-base-Malicious_URLs",
    "laiyer/codebert-base-Malicious_URLs-onnx",  # ONNX version
)

_malicious_labels = [
    "defacement",
    "phishing",
    "malware",
]

# URL pattern
url_pattern = re.compile(
    r"http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\\(\\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+"
)


class MaliciousURLs(Scanner):
    """
    This scanner is used to scan and detect malicious URLs in the text.

    Large Language Models can occasionally produce malicious URLs in their outputs.
    Using the "DunnBC22/codebert-base-Malicious_URLs" model from HuggingFace, this class classifies
    URLs as either malicious or benign to safeguard users from potential threats.
    """

    def __init__(self, threshold=0.5, use_onnx: bool = False):
        """
        Initializes an instance of the MaliciousURLs class.

        Parameters:
            threshold (float): The threshold used to determine if the website is malicious. Defaults to 0.5.
            use_onnx (bool): Whether to use the ONNX version of the model. Defaults to False.
        """

        self._threshold = threshold
        self._classifier = pipeline_text_classification(
            model=_model_path[0],
            onnx_model=_model_path[1],
            truncation=True,
            use_onnx=use_onnx,
            top_k=None,
            max_length=512,
        )

    @staticmethod
    def extract_urls(text: str) -> List[str]:
        return url_pattern.findall(text)

    def scan(self, prompt: str, output: str) -> (str, bool, float):
        if prompt.strip() == "":
            return output, True, 0.0

        urls = self.extract_urls(output)
        if len(urls) == 0:
            return output, True, 0.0

        logger.debug(f"Found {len(urls)} URLs in the output")
        results = self._classifier(urls)
        for url, result in zip(urls, results):
            malicious_scores = [
                item["score"] for item in result if item["label"] in _malicious_labels
            ]
            highest_malicious_score = max(malicious_scores)
            if highest_malicious_score > self._threshold:
                logger.warning(
                    f"Detected malware URL '{url}' with score: {highest_malicious_score}"
                )

                return output, False, calculate_risk_score(highest_malicious_score, self._threshold)

        logger.debug(f"Not malware URLs in the output: {results}")

        return output, True, 0.0



import os
import re
from typing import List

from llm_guard.util import logger

from .base import Scanner

stop_file_path = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    "..",
    "resources",
    "prompt_stop_substrings.json",
)

MATCH_TYPE_STR = "str"
MATCH_TYPE_WORD = "word"
allowed_match_type = [MATCH_TYPE_STR, MATCH_TYPE_WORD]


class BanSubstrings(Scanner):
    """
    BanSubstrings class is used to ban certain substrings from appearing in the prompt.

    The match can be done either at a string level or word level.
    """

    def __init__(
        self,
        match_type: str = MATCH_TYPE_STR,
        case_sensitive: bool = False,
        substrings: List[str] = None,
        redact: bool = False,
        contains_all: bool = False,  # contains any
    ):
        """
        Initialize BanSubstrings object.

        Args:
            match_type (str, optional): The type of match to be performed. Either 'str' or 'word'.
                                        'str' means the match is performed at the string level and 'word' means the match is done at the word level.
                                        Default is 'str'.
            case_sensitive (bool, optional): Flag to indicate if the match should be case-sensitive. Default is False.
            substrings (List[str], optional): List of substrings to ban.
            redact (bool, optional): Flag to indicate if the banned substrings should be redacted. Default is False.
            contains_all (bool): Flag to indicate if need to match all substrings instead of any of them. Default is contains any.

        Raises:
            ValueError: If no substrings are provided or match_type is not 'str' or 'word'.
        """

        if not substrings:
            raise ValueError("No substrings provided")

        if match_type not in allowed_match_type:
            raise ValueError(f"This match_type is not recognized. Allowed: {allowed_match_type}")

        self._match_type = match_type  # str or word
        self._case_sensitive = case_sensitive
        self._substrings = substrings
        self._redact = redact
        self._contains_all = contains_all

    @staticmethod
    def _redact_text(text: str, substrings: List[str]) -> str:
        redacted_text = text
        for s in substrings:
            redacted_text = redacted_text.replace(s, "[REDACTED]")
        return redacted_text

    def scan(self, prompt: str) -> (str, bool, float):
        matched_substrings = []
        missing_substrings = []
        for s in self._substrings:
            if self._case_sensitive:
                s, prompt = s.lower(), prompt.lower()

            if self._match_type == MATCH_TYPE_STR:
                if s in prompt:
                    matched_substrings.append(s)
                else:
                    missing_substrings.append(s)

            if self._match_type == MATCH_TYPE_WORD:
                if re.search(r"\b" + s + r"\b", prompt):
                    matched_substrings.append(s)
                else:
                    missing_substrings.append(s)

        if self._contains_all:
            if len(missing_substrings) > 0:
                logger.debug(f"Some substrings were not found: " + ", ".join(missing_substrings))
                return prompt, True, 0.0

            if self._redact:
                prompt = self._redact_text(prompt, matched_substrings)
                logger.debug("Redacted banned substrings")

            logger.warning(f"All substrings were found")

            return prompt, False, 1.0

        if matched_substrings:
            logger.warning(
                f"Found the following banned substrings: " + ", ".join(matched_substrings)
            )

            if self._redact:
                prompt = self._redact_text(prompt, matched_substrings)
                logger.debug("Redacted banned substrings")

            return prompt, False, 1.0

        logger.debug("No banned substrings found")

        return prompt, True, 0.0



"""
This plugin searches for LinkedIn Client IDs and LinkedIn Client secrets.
"""

import re

from detect_secrets.plugins.base import RegexBasedDetector


class LinkedInDetector(RegexBasedDetector):
    """Scans for LinkedIn secrets."""

    secret_type = "LinkedIn Secret"

    denylist = [
        # LinkedIn Client ID
        re.compile(
            r"""(?i)(?:linkedin|linked-in)(?:[0-9a-z\-_\t .]{0,20})(?:[\s|']|[\s|"]){0,3}(?:=|>|:{1,3}=|\|\|:|<=|=>|:|\?=)(?:'|\"|\s|=|\x60){0,5}([a-z0-9]{14})(?:['|\"|\n|\r|\s|\x60|;]|$)"""
        ),
        # LinkedIn Client secret
        re.compile(
            r"""(?i)(?:linkedin|linked-in)(?:[0-9a-z\-_\t .]{0,20})(?:[\s|']|[\s|"]){0,3}(?:=|>|:{1,3}=|\|\|:|<=|=>|:|\?=)(?:'|\"|\s|=|\x60){0,5}([a-z0-9]{16})(?:['|\"|\n|\r|\s|\x60|;]|$)"""
        ),
    ]



from llm_guard.input_scanners.ban_topics import MODEL_LARGE
from llm_guard.transformers_helpers import pipeline_zero_shot_classification
from llm_guard.util import logger

from .base import Scanner

_model = MODEL_LARGE
_categories = ["refusal", "not_refusal"]


class NoRefusal(Scanner):
    """
    A class used to detect and handle refusals in language model output.

    This class uses the zero-shot-classification model from Hugging Face to classify refusals.

    Refusals are common when the prompt breaches policies defined by the model.
    """

    def __init__(self, threshold: float = 0.5, use_onnx: bool = False):
        """
        Initializes an instance of the NoRefusal class.

        Parameters:
            threshold (float): The similarity threshold to consider an output as refusal.
            use_onnx (bool): Whether to use the ONNX version of the model. Defaults to False.
        """

        self._threshold = threshold

        self._classifier = pipeline_zero_shot_classification(
            model=_model["path"],
            onnx_model=_model["onnx_path"],
            use_onnx=use_onnx,
            max_length=_model["max_length"],
            truncation=True,
        )

    def scan(self, prompt: str, output: str) -> (str, bool, float):
        if output.strip() == "":
            return output, True, 0.0

        output_model = self._classifier(output, _categories, multi_label=True)
        combined_output = dict(zip(output_model["labels"], output_model["scores"]))

        max_score = round(combined_output["refusal"], 2)
        if max_score > self._threshold:
            logger.warning(f"Refusal detected for the prompt: {combined_output}")

            return output, False, max_score

        logger.debug(f"No refusal detected: {combined_output}")

        return output, True, 0.0



from typing import List

from llm_guard.transformers_helpers import pipeline_text_classification
from llm_guard.util import calculate_risk_score, logger

from .base import Scanner

model_path = (
    "papluca/xlm-roberta-base-language-detection",
    "laiyer/xlm-roberta-base-language-detection-onnx",
)


class Language(Scanner):
    """
    A Scanner subclass responsible for determining the language of a given text
    prompt and verifying its validity against a list of predefined languages.
    """

    def __init__(
        self,
        valid_languages: List[str],
        threshold: float = 0.6,
        use_onnx: bool = False,
    ):
        """
        Initializes the Language scanner with a list of valid languages.

        Parameters:
            valid_languages (List[str]): A list of valid language codes in ISO 639-1.
            threshold (float): Minimum confidence score
            use_onnx (bool): Whether to use ONNX for inference. Default is False.
        """

        self._threshold = threshold
        self._valid_languages = valid_languages
        self._pipeline = pipeline_text_classification(
            model=model_path[0],
            onnx_model=model_path[1],
            top_k=None,
            use_onnx=use_onnx,
            truncation=True,
            max_length=512,
        )

    def scan(self, prompt: str) -> (str, bool, float):
        if prompt.strip() == "":
            return prompt, True, 0.0

        highest_score = 0.0
        results = self._pipeline(prompt)
        for result in results[0]:
            if result["label"] in self._valid_languages:
                if result["score"] > self._threshold:
                    logger.debug(
                        f"Language {result['label']} is found in the text with confidence: {result['score']}"
                    )

                    return prompt, True, 0.0

                if result["score"] > highest_score:
                    highest_score = result["score"]

                logger.warning(
                    f"Language {result['label']} is found but confidence is too low: {result['score']}"
                )

        return prompt, False, calculate_risk_score(highest_score, self._threshold)
